@@ -17,7 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
-
 #include <gtk/gtk.h>
 #include <NvCtrlAttributes.h>
 #include <string.h>
@@ -37,7 +36,7 @@
 #include "ctkhelp.h"
 #include "ctkutils.h"
 
-static void ctk_display_device_class_init(CtkDisplayDeviceClass *);
+static void ctk_display_device_class_init(CtkDisplayDeviceClass *, gpointer);
 static void ctk_display_device_finalize(GObject *);
 
 static void reset_button_clicked(GtkButton *button, gpointer user_data);
@@ -245,7 +244,8 @@ GType ctk_display_device_get_type(void)
 }
 
 static void ctk_display_device_class_init(
-    CtkDisplayDeviceClass *ctk_object_class
+    CtkDisplayDeviceClass *ctk_object_class,
+    gpointer class_data
 )
 {
     GObjectClass *gobject_class = (GObjectClass *)ctk_object_class;
@@ -281,7 +281,7 @@ static void ctk_display_device_finalize(
 
 
 /*
- * ctk_display_device_new() - constructor for the dissplay device page.
+ * ctk_display_device_new() - constructor for the display device page.
  */
 
 GtkWidget* ctk_display_device_new(CtrlTarget *ctrl_target,
@@ -924,8 +924,6 @@ static gboolean update_vrr_enabled_info(InfoEntry *entry)
 
 
 
-
-
 static gboolean update_refresh_rate(InfoEntry *entry)
 {
     CtkDisplayDevice *ctk_object = entry->ctk_object;
