@@ -44,12 +44,6 @@ static void powermizer_menu_changed(GtkWidget*, gpointer);
 static void update_powermizer_menu_event(GObject *object,
                                          gpointer arg1,
                                          gpointer user_data);
-static void dp_config_button_toggled(GtkWidget *, gpointer);
-static void dp_set_config_status(CtkPowermizer *);
-static void dp_update_config_status(CtkPowermizer *, gboolean);
-static void dp_configuration_update_received(GObject *, CtrlEvent *, gpointer);
-static void post_dp_configuration_update(CtkPowermizer *);
-static void show_dp_toggle_warning_dlg(CtkPowermizer *ctk_powermizer);
 static void post_set_attribute_offset_value(CtkPowermizer *ctk_powermizer,
                                             gint attribute,
                                             gint val);
@@ -87,13 +81,10 @@ static const char *__gpu_clock_freq_help =
 static const char *__memory_transfer_rate_freq_help =
 "This indicates the current Memory transfer rate.";
 
-static const char *__processor_clock_freq_help =
-"This indicates the current Processor Clock frequency.";
-
 static const char *__performance_levels_table_help =
 "This indicates the Performance Levels available for the GPU.  Each "
 "performance level is indicated by a Performance Level number, along with "
-"the Graphics, Memory and Processor clocks for that level.  The currently active "
+"the Graphics and Memory clocks for that level.  The currently active "
 "performance level is shown in regular text.  All other performance "
 "levels are shown in gray.  Note that multiple performance levels may share "
 "the same range of available clocks.";
@@ -134,13 +125,6 @@ static const char *__powermizer_prefer_consistent_performance_help =
 "'Prefer Consistent Performance' hints to the driver to lock to GPU base clocks, "
 "when possible.  ";
 
-static const char *__dp_configuration_button_help =
-"CUDA - Double Precision lets you enable "
-"increased double-precision calculations in CUDA applications.  Available on "
-"GPUs with the capability for increased double-precision performance."
-"  NOTE: Selecting a GPU reduces performance for non-CUDA applications, "
-"including games.  To increase game performance, disable this checkbox.";
-
 GType ctk_powermizer_get_type(void)
 {
     static GType ctk_powermizer_type = 0;
@@ -175,8 +159,6 @@ typedef struct {
     gboolean perf_level_specified;
     gint nvclock;
     gboolean nvclock_specified;
-    gint processorclock;
-    gboolean processorclock_specified;
     gint nvclockmin;
     gboolean nvclockmin_specified;
     gint nvclockmax;
@@ -189,11 +171,6 @@ typedef struct {
     gint memtransferratemax;
     gint memtransferrateeditable;
     gboolean memtransferratemax_specified;
-    gint processorclockmin;
-    gboolean processorclockmin_specified;
-    gint processorclockmax;
-    gint processorclockeditable;
-    gboolean processorclockmax_specified;
 } perfModeEntry, * perfModeEntryPtr;
 
 
@@ -226,17 +203,6 @@ static void apply_perf_mode_token(char *token, char *value, void *data)
         pEntry->memtransferratemax_specified = TRUE;
     } else if (!strcasecmp("memtransferrateeditable", token)) {
         pEntry->memtransferrateeditable = atoi(value);
-    } else if (!strcasecmp("processorclock", token)) {
-        pEntry->processorclock = atoi(value);
-        pEntry->processorclock_specified = TRUE;
-    } else if (!strcasecmp("processorclockmin", token)) {
-        pEntry->processorclockmin = atoi(value);
-        pEntry->processorclockmax_specified = TRUE;
-    } else if (!strcasecmp("processorclockmax", token)) {
-        pEntry->processorclockmax = atoi(value);
-        pEntry->processorclockmax_specified = TRUE;
-    } else if (!strcasecmp("processorclockeditable", token)) {
-        pEntry->nvclockeditable = atoi(value);
     }
 }
 
@@ -672,9 +638,7 @@ static void update_perf_mode_table(CtkPowermizer *ctk_powermizer,
             if (!ctk_powermizer->hasDecoupledClock &&
                 ((pEntry[index].nvclockmax != pEntry[index].nvclockmin) ||
                  (pEntry[index].memtransferratemax !=
-                  pEntry[index].memtransferratemin) ||
-                 (pEntry[index].processorclockmax !=
-                  pEntry[index].processorclockmin))) {
+                  pEntry[index].memtransferratemin))) {
                 ctk_powermizer->hasDecoupledClock = TRUE;
             }
             /* Set hasEditablePerfLevel flag to decide editable performance
@@ -780,27 +744,6 @@ static void update_perf_mode_table(CtkPowermizer *ctk_powermizer,
             col_idx += 4;
         }
 
-        if (ctk_powermizer->processor_clock) {
-            /* Processor clock */
-            label = gtk_label_new("Processor Clock");
-            gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-            gtk_table_attach(GTK_TABLE(table), label, col_idx+1, col_idx+3, 0, 1,
-                             GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-            label = gtk_label_new("Min");
-            gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-            gtk_table_attach(GTK_TABLE(table), label, col_idx+1, col_idx+2, 1, 2,
-                             GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-            label = gtk_label_new("Max");
-            gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-            gtk_table_attach(GTK_TABLE(table), label, col_idx+2, col_idx+3, 1, 2,
-                             GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-
-            /* Vertical separator */
-            vsep = gtk_vseparator_new();
-            gtk_table_attach(GTK_TABLE(table), vsep, col_idx+3, col_idx+4,
-                             0, row_idx,
-                             GTK_FILL, GTK_FILL | GTK_EXPAND, 0, 0);
-        }
     } else {
 
         table = gtk_table_new(1, 4, FALSE);
@@ -834,13 +777,6 @@ static void update_perf_mode_table(CtkPowermizer *ctk_powermizer,
                              GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
             col_idx++;
         }
-
-        if (ctk_powermizer->processor_clock) {
-            label = gtk_label_new("Processor Clock");
-            gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-            gtk_table_attach(GTK_TABLE(table), label, col_idx+1, col_idx+2, 0, 1,
-                             GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-        }
     }
 
     /* Parse the perf levels and populate the table */
@@ -905,22 +841,6 @@ static void update_perf_mode_table(CtkPowermizer *ctk_powermizer,
                                  GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
                 col_idx +=4;
             }
-            if (ctk_powermizer->processor_clock) {
-                g_snprintf(tmp_str, 24, "%d MHz", pEntry[i].processorclockmin);
-                label = gtk_label_new(tmp_str);
-                gtk_widget_set_sensitive(label, active);
-                gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-                gtk_table_attach(GTK_TABLE(table), label, col_idx+1, col_idx+2,
-                                 row_idx, row_idx+1,
-                                 GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-                g_snprintf(tmp_str, 24, "%d MHz", pEntry[i].processorclockmax);
-                label = gtk_label_new(tmp_str);
-                gtk_widget_set_sensitive(label, active);
-                gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-                gtk_table_attach(GTK_TABLE(table), label, col_idx+2, col_idx+3,
-                                 row_idx, row_idx+1,
-                                 GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-            }
             row_idx++;
         } else if ((pEntry[i].perf_level_specified) &&
                    (pEntry[i].nvclock_specified)) {
@@ -964,15 +884,6 @@ static void update_perf_mode_table(CtkPowermizer *ctk_powermizer,
                                  GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
                 col_idx++;
             }
-            if (ctk_powermizer->processor_clock) {
-                g_snprintf(tmp_str, 24, "%d MHz", pEntry[i].processorclock);
-                label = gtk_label_new(tmp_str);
-                gtk_widget_set_sensitive(label, active);
-                gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-                gtk_table_attach(GTK_TABLE(table), label, col_idx, col_idx+1,
-                                 row_idx, row_idx+1,
-                                 GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-            }
             row_idx++;
         } else {
             nv_warning_msg("Incomplete Perf Mode (perf=%d, nvclock=%d,"
@@ -1050,12 +961,6 @@ static gboolean update_powermizer_info(gpointer user_data)
             gtk_label_set_text(GTK_LABEL(ctk_powermizer->memory_transfer_rate), s);
             g_free(s);
         }
-
-        if (ctk_powermizer->processor_clock && pEntry.processorclock_specified) {
-            s = g_strdup_printf("%d Mhz", pEntry.processorclock);
-            gtk_label_set_text(GTK_LABEL(ctk_powermizer->processor_clock), s);
-            g_free(s);
-        }
     }
     free(clock_string);
 
@@ -1077,13 +982,15 @@ static gboolean update_powermizer_info(gpointer user_data)
         g_free(s);
     }
 
-    if (ctk_powermizer->pcie_gen_queriable) {
+    if (ctk_powermizer->link_width) {
         /* NV_CTRL_GPU_PCIE_CURRENT_LINK_WIDTH */
         s = get_pcie_link_width_string(ctrl_target,
                                        NV_CTRL_GPU_PCIE_CURRENT_LINK_WIDTH);
         gtk_label_set_text(GTK_LABEL(ctk_powermizer->link_width), s);
         g_free(s);
+    }
 
+    if (ctk_powermizer->link_speed) {
         /* NV_CTRL_GPU_PCIE_MAX_LINK_SPEED */
         s = get_pcie_link_speed_string(ctrl_target,
                                        NV_CTRL_GPU_PCIE_CURRENT_LINK_SPEED);
@@ -1190,21 +1097,19 @@ GtkWidget* ctk_powermizer_new(CtrlTarget *ctrl_target,
     GtkWidget *hbox, *hbox2, *vbox, *vbox2, *hsep, *table;
     GtkWidget *banner, *label;
     CtkDropDownMenu *menu;
-    ReturnStatus ret, ret1;
-    gint attribute;
+    ReturnStatus ret;
     gint nvclock_attribute = 0, mem_transfer_rate_attribute = 0;
     gint val;
     gint row = 0;
     gchar *s = NULL;
     gint tmp;
-    gboolean processor_clock_available = FALSE;
     gboolean power_source_available = FALSE;
     gboolean perf_level_available = FALSE;
     gboolean gpu_clock_available = FALSE;
     gboolean mem_transfer_rate_available = FALSE;
     gboolean adaptive_clock_state_available = FALSE;
-    gboolean cuda_dp_ui = FALSE;
-    gboolean pcie_gen_queriable = FALSE;
+    gboolean pcie_link_width_available = FALSE;
+    gboolean pcie_link_speed_available = FALSE;
     CtrlAttributeValidValues valid_modes;
     char *clock_string = NULL;
     perfModeEntry pEntry;
@@ -1256,23 +1161,27 @@ GtkWidget* ctk_powermizer_new(CtrlTarget *ctrl_target,
         if (pEntry.memtransferrate_specified) {
             mem_transfer_rate_available = TRUE;
         }
-        if (pEntry.processorclock_specified) {
-            processor_clock_available = TRUE;
-        }
     }
     free(clock_string);
 
     /* NV_CTRL_GPU_PCIE_GENERATION */
 
-    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GPU_PCIE_GENERATION, &tmp);
+    ret = NvCtrlGetAttribute(ctrl_target,
+                             NV_CTRL_GPU_PCIE_CURRENT_LINK_WIDTH,
+                             &tmp);
     if (ret == NvCtrlSuccess) {
-        pcie_gen_queriable = TRUE;
+        pcie_link_width_available = TRUE;
+    }
+
+    ret = NvCtrlGetAttribute(ctrl_target,
+                             NV_CTRL_GPU_PCIE_CURRENT_LINK_SPEED,
+                             &tmp);
+    if (ret == NvCtrlSuccess) {
+        pcie_link_speed_available = TRUE;
     }
 
     /* return early if query to attributes fail */
-    if (!power_source_available && !perf_level_available &&
-        !adaptive_clock_state_available && !gpu_clock_available &&
-        !processor_clock_available && !pcie_gen_queriable) {
+    if (!perf_level_available && !gpu_clock_available) {
         return NULL;
     }
     /* create the CtkPowermizer object */
@@ -1282,7 +1191,6 @@ GtkWidget* ctk_powermizer_new(CtrlTarget *ctrl_target,
     ctk_powermizer = CTK_POWERMIZER(object);
     ctk_powermizer->ctrl_target = ctrl_target;
     ctk_powermizer->ctk_config = ctk_config;
-    ctk_powermizer->pcie_gen_queriable = pcie_gen_queriable;
     ctk_powermizer->hasDecoupledClock = FALSE;
     ctk_powermizer->hasEditablePerfLevel = FALSE;
     ctk_powermizer->editable_performance_levels_unified = FALSE;
@@ -1399,24 +1307,6 @@ GtkWidget* ctk_powermizer_new(CtrlTarget *ctrl_target,
     } else {
         ctk_powermizer->memory_transfer_rate = NULL;
     }
-    /* Processor clock */
-    if (processor_clock_available) {
-        /* spacing */
-        row += 3;
-        ctk_powermizer->processor_clock =
-            add_table_row_with_help_text(table, ctk_config,
-                                         __processor_clock_freq_help,
-                                         row++, //row
-                                         0,  // column
-                                         0.0f,
-                                         0.5,
-                                         "Processor Clock:",
-                                         0.0,
-                                         0.5,
-                                         NULL);
-    } else {
-        ctk_powermizer->processor_clock = NULL;
-    }
     /* Power Source */
     if (power_source_available) {
         /* spacing */
@@ -1435,10 +1325,13 @@ GtkWidget* ctk_powermizer_new(CtrlTarget *ctrl_target,
     } else {
         ctk_powermizer->power_source = NULL;
     }
-    /* Current PCIe Link Width */
-    if (ctk_powermizer->pcie_gen_queriable) {
+    /* PCIe Gen Info block */
+    if (pcie_link_width_available || pcie_link_speed_available) {
         /* spacing */
         row += 3;
+    }
+    /* Current PCIe Link Width */
+    if (pcie_link_width_available) {
         ctk_powermizer->link_width =
             add_table_row_with_help_text(table, ctk_config,
                                          __current_pcie_link_width_help,
@@ -1451,7 +1344,11 @@ GtkWidget* ctk_powermizer_new(CtrlTarget *ctrl_target,
                                          0.5,
                                          NULL);
 
-        /* Current PCIe Link Speed */
+    } else {
+        ctk_powermizer->link_width = NULL;
+    }
+    /* Current PCIe Link Speed */
+    if (pcie_link_speed_available) {
         ctk_powermizer->link_speed =
             add_table_row_with_help_text(table, ctk_config,
                                          __current_pcie_link_speed_help,
@@ -1464,7 +1361,6 @@ GtkWidget* ctk_powermizer_new(CtrlTarget *ctrl_target,
                                          0.5,
                                          NULL);
     } else {
-        ctk_powermizer->link_width = NULL;
         ctk_powermizer->link_speed = NULL;
     }
 
@@ -1622,109 +1518,6 @@ GtkWidget* ctk_powermizer_new(CtrlTarget *ctrl_target,
         update_powermizer_menu_info(ctk_powermizer);
     }
 
-    /*
-     * check if CUDA - Double Precision Boost support available.
-     */
-
-    ret = NvCtrlGetAttribute(ctrl_target,
-                             NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE,
-                             &val);
-    if (ret == NvCtrlSuccess) {
-        attribute = NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE;
-        cuda_dp_ui = TRUE;
-    } else {
-        ret1 = NvCtrlGetAttribute(ctrl_target,
-                                  NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT,
-                                  &val);
-        if (ret1 == NvCtrlSuccess) {
-            attribute = NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT;
-            cuda_dp_ui = TRUE;
-        }
-    }
-
-    if (cuda_dp_ui) {
-        ctk_powermizer->attribute = attribute;
-        ctk_powermizer->dp_toggle_warning_dlg_shown = FALSE;
-
-        /* Query CUDA - Double Precision Boost Status */
-
-        dp_update_config_status(ctk_powermizer, val);
-
-        /* CUDA - Double Precision Boost configuration settings */
-
-        hbox = gtk_hbox_new(FALSE, 0);
-        gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-
-        label = gtk_label_new("CUDA");
-        gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-        hsep = gtk_hseparator_new();
-        gtk_box_pack_start(GTK_BOX(hbox), hsep, TRUE, TRUE, 5);
-
-        hbox2 = gtk_hbox_new(FALSE, 0);
-        ctk_powermizer->configuration_button =
-            gtk_check_button_new_with_label("CUDA - Double precision");
-        gtk_box_pack_start(GTK_BOX(hbox2),
-                           ctk_powermizer->configuration_button,
-                           FALSE, FALSE, 0);
-        gtk_container_set_border_width(GTK_CONTAINER(hbox2), 0);
-        gtk_toggle_button_set_active
-            (GTK_TOGGLE_BUTTON(ctk_powermizer->configuration_button),
-             ctk_powermizer->dp_enabled);
-
-        /* Packing */
-
-        table = gtk_table_new(1, 1, FALSE);
-        gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
-        gtk_table_set_row_spacings(GTK_TABLE(table), 3);
-        gtk_table_set_col_spacings(GTK_TABLE(table), 15);
-        gtk_container_set_border_width(GTK_CONTAINER(table), 5);
-
-        gtk_table_attach(GTK_TABLE(table), hbox2, 0, 1, 0, 1,
-                         GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-
-        if (attribute == NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT) {
-            GtkWidget *separator;
-
-            gtk_table_resize(GTK_TABLE(table), 1, 3);
-            /* V-bar */
-            hbox2 = gtk_hbox_new(FALSE, 0);
-            separator = gtk_vseparator_new();
-            gtk_box_pack_start(GTK_BOX(hbox2), separator, FALSE, FALSE, 0);
-            gtk_table_attach(GTK_TABLE(table), hbox2, 1, 2, 0, 1,
-                             GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-
-            ctk_powermizer->status = gtk_label_new("");
-            gtk_misc_set_alignment(GTK_MISC(ctk_powermizer->status), 0.0f, 0.5f);
-            hbox2 = gtk_hbox_new(FALSE, 0);
-            gtk_box_pack_start(GTK_BOX(hbox2),
-                               ctk_powermizer->status, FALSE, FALSE, 0);
-
-            gtk_table_attach(GTK_TABLE(table), hbox2, 2, 3, 0, 1,
-                             GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
-        }
-
-        ctk_config_set_tooltip(ctk_config, ctk_powermizer->configuration_button,
-                               __dp_configuration_button_help);
-        g_signal_connect(G_OBJECT(ctk_powermizer->configuration_button),
-                         "clicked",
-                         G_CALLBACK(dp_config_button_toggled),
-                         (gpointer) ctk_powermizer);
-        if (attribute == NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE) {
-            g_signal_connect(G_OBJECT(ctk_event),
-                   CTK_EVENT_NAME(NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE),
-                   G_CALLBACK(dp_configuration_update_received),
-                   (gpointer) ctk_powermizer);
-        } else if (attribute == NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT) {
-            g_signal_connect(G_OBJECT(ctk_event),
-                   CTK_EVENT_NAME(NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT),
-                   G_CALLBACK(dp_configuration_update_received),
-                   (gpointer) ctk_powermizer);
-        }
-    } else {
-        ctk_powermizer->configuration_button = NULL;
-    }
-
     /* Updating the powermizer page */
 
     update_powermizer_info(ctk_powermizer);
@@ -1891,170 +1684,6 @@ static void powermizer_menu_changed(GtkWidget *widget,
 
 
 
-static void show_dp_toggle_warning_dlg(CtkPowermizer *ctk_powermizer)
-{
-    GtkWidget *dlg, *parent;
-
-    /* return early if message dialog already shown */
-    if (ctk_powermizer->dp_toggle_warning_dlg_shown) {
-        return;
-    }
-    ctk_powermizer->dp_toggle_warning_dlg_shown = TRUE;
-    parent = ctk_get_parent_window(GTK_WIDGET(ctk_powermizer));
-
-    dlg = gtk_message_dialog_new (GTK_WINDOW(parent),
-                                  GTK_DIALOG_MODAL,
-                                  GTK_MESSAGE_WARNING,
-                                  GTK_BUTTONS_OK,
-                                  "Changes to the CUDA - Double precision "
-                                  "setting "
-                                  "require a system reboot before "
-                                  "taking effect.");
-    gtk_dialog_run(GTK_DIALOG(dlg));
-    gtk_widget_destroy (dlg);
-
-} /* show_dp_toggle_warning_dlg() */
-
-
-
-/*
- * post_dp_configuration_update() - this function updates status bar string.
- */
-
-static void post_dp_configuration_update(CtkPowermizer *ctk_powermizer)
-{
-    gboolean enabled = ctk_powermizer->dp_enabled;
-
-    const char *conf_string = enabled ? "enabled" : "disabled";
-    char message[128];
-
-    if (ctk_powermizer->attribute == NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT) {
-        snprintf(message, sizeof(message), "CUDA - Double precision will "
-                 "be %s after reboot.",
-                 conf_string);
-    } else {
-        snprintf(message, sizeof(message), "CUDA - Double precision %s.",
-                 conf_string);
-    }
-
-    ctk_config_statusbar_message(ctk_powermizer->ctk_config, "%s", message);
-} /* post_dp_configuration_update() */
-
-
-
-/*
- * dp_set_config_status() - set CUDA - Double Precision Boost configuration
- * button status.
- */
-
-static void dp_set_config_status(CtkPowermizer *ctk_powermizer)
-{
-    GtkWidget *configuration_button = ctk_powermizer->configuration_button;
-
-    g_signal_handlers_block_by_func(G_OBJECT(configuration_button),
-                                    G_CALLBACK(dp_config_button_toggled),
-                                    (gpointer) ctk_powermizer);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(configuration_button),
-                                 ctk_powermizer->dp_enabled);
-
-    g_signal_handlers_unblock_by_func(G_OBJECT(configuration_button),
-                                      G_CALLBACK(dp_config_button_toggled),
-                                      (gpointer) ctk_powermizer);
-} /* dp_set_config_status() */
-
-
-
-/*
- * dp_update_config_status - get current CUDA - Double Precision Boost status.
- */
-
-static void dp_update_config_status(CtkPowermizer *ctk_powermizer, gboolean val)
-{
-    if ((ctk_powermizer->attribute ==
-         NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE &&
-         val == NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE_DISABLED) ||
-        (ctk_powermizer->attribute ==
-         NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT &&
-         val == NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT_DISABLED)) {
-        ctk_powermizer->dp_enabled = FALSE;
-    } else {
-        ctk_powermizer->dp_enabled = TRUE;
-    }
-} /* dp_update_config_status() */
-
-
-
-/*
- * dp_configuration_update_received() - this function is called when the
- * NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE attribute is changed by another
- * NV-CONTROL client.
- */
-
-static void dp_configuration_update_received(GObject *object,
-                                             CtrlEvent *event,
-                                             gpointer user_data)
-{
-    CtkPowermizer *ctk_powermizer = CTK_POWERMIZER(user_data);
-
-    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
-        return;
-    }
-
-    ctk_powermizer->dp_enabled = event->int_attr.value;
-
-    /* set CUDA - Double Precision Boost configuration buttion status */
-    dp_set_config_status(ctk_powermizer);
-
-    /* Update status bar message */
-    post_dp_configuration_update(ctk_powermizer);
-} /* dp_configuration_update_received() */
-
-
-/*
- * dp_config_button_toggled() - callback function for
- * enable CUDA - Double Precision Boost checkbox.
- */
-
-static void dp_config_button_toggled(GtkWidget *widget,
-                                     gpointer user_data)
-{
-    gboolean enabled;
-    CtkPowermizer *ctk_powermizer = CTK_POWERMIZER(user_data);
-    CtrlTarget *ctrl_target = ctk_powermizer->ctrl_target;
-    ReturnStatus ret;
-
-    enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-
-    /* show popup dialog when user first time click DP config */
-    if (ctk_powermizer->attribute == NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT) {
-        show_dp_toggle_warning_dlg(ctk_powermizer);
-    }
-
-    /* set the newly specified CUDA - Double Precision Boost value */
-    ret = NvCtrlSetAttribute(ctrl_target,
-                             ctk_powermizer->attribute,
-                             enabled);
-    if (ret != NvCtrlSuccess) {
-        ctk_config_statusbar_message(ctk_powermizer->ctk_config,
-                                     "Failed to set "
-                                     "CUDA - Double precision "
-                                     "configuration!");
-        return;
-    }
-
-    ctk_powermizer->dp_enabled = enabled;
-    dp_set_config_status(ctk_powermizer);
-    if (ctk_powermizer->status) {
-        gtk_label_set_text(GTK_LABEL(ctk_powermizer->status),
-                           "pending reboot");
-    }
-
-    /* Update status bar message */
-    post_dp_configuration_update(ctk_powermizer);
-} /* dp_config_button_toggled() */
-
-
-
 GtkTextBuffer *ctk_powermizer_create_help(GtkTextTagTable *table,
                                           CtkPowermizer *ctk_powermizer)
 {
@@ -2077,11 +1706,7 @@ GtkTextBuffer *ctk_powermizer_create_help(GtkTextTagTable *table,
 
     if (ctk_powermizer->gpu_clock) {
         ctk_help_heading(b, &i, "Clock Frequencies");
-        if (ctk_powermizer->memory_transfer_rate && 
-            ctk_powermizer->processor_clock) {
-            s = "This indicates the GPU's current Graphics Clock, "
-                "Memory transfer rate and Processor Clock frequencies.";
-        } else if (ctk_powermizer->memory_transfer_rate) {
+        if (ctk_powermizer->memory_transfer_rate) {
             s = "This indicates the GPU's current Graphics Clock and "
                 "Memory transfer rate.";
         } else {
@@ -2095,9 +1720,11 @@ GtkTextBuffer *ctk_powermizer_create_help(GtkTextTagTable *table,
         ctk_help_para(b, &i, "%s", __power_source_help);
     }
 
-    if (ctk_powermizer->pcie_gen_queriable) {
+    if (ctk_powermizer->link_width) {
         ctk_help_heading(b, &i, "Current PCIe link width");
         ctk_help_para(b, &i, "%s", __current_pcie_link_width_help);
+    }
+    if (ctk_powermizer->link_speed) {
         ctk_help_heading(b, &i, "Current PCIe link speed");
         ctk_help_para(b, &i, "%s", __current_pcie_link_speed_help);
     }
@@ -2123,11 +1750,6 @@ GtkTextBuffer *ctk_powermizer_create_help(GtkTextTagTable *table,
         ctk_help_para(b, &i, "%s", ctk_powermizer->powermizer_menu_help);
     }
 
-    if (ctk_powermizer->configuration_button) {
-        ctk_help_heading(b, &i, "CUDA - Double precision");
-        ctk_help_para(b, &i, "%s", __dp_configuration_button_help);
-    }
-
     ctk_help_finish(b);
 
     return b;
