@@ -69,22 +69,6 @@ const CtrlTargetTypeInfo targetTypeInfoTable[] = {
       NV_FALSE,                                                      /* uses_display_devices */
       1, 10 },                                                       /* required major,minor protocol rev */
 
-    [VCS_TARGET] =
-    { "VCS",                                                         /* name */
-      "vcs",                                                         /* parsed_name */
-      NV_CTRL_TARGET_TYPE_VCSC,                                      /* nvctrl */
-      CTRL_TARGET_PERM_BIT(VCS_TARGET),                              /* permission_bit */
-      NV_FALSE,                                                      /* uses_display_devices */
-      1, 12 },                                                       /* required major,minor protocol rev */
-
-    [GVI_TARGET] =
-    { "SDI Input Device",                                            /* name */
-      "gvi",                                                         /* parsed_name */
-      NV_CTRL_TARGET_TYPE_GVI,                                       /* nvctrl */
-      CTRL_TARGET_PERM_BIT(GVI_TARGET),                              /* permission_bit */
-      NV_FALSE,                                                      /* uses_display_devices */
-      1, 18 },                                                       /* required major,minor protocol rev */
-
     [COOLER_TARGET] =
     { "Fan",                                                         /* name */
       "fan",                                                         /* parsed_name */
@@ -116,6 +100,14 @@ const CtrlTargetTypeInfo targetTypeInfoTable[] = {
       CTRL_TARGET_PERM_BIT(DISPLAY_TARGET),                          /* permission_bit */
       NV_FALSE,                                                      /* uses_display_devices */
       1, 27 },                                                       /* required major,minor protocol rev */
+
+    [MUX_TARGET] =
+    { "Mux Device",                                                  /* name */
+      "mux",                                                         /* parsed_name */
+      NV_CTRL_TARGET_TYPE_MUX,                                       /* nvctrl */
+      CTRL_TARGET_PERM_BIT(MUX_TARGET),                              /* permission_bit */
+      NV_FALSE,                                                      /* uses_display_devices */
+      1, 28 },                                                       /* required major,minor protocol rev */
 };
 
 const int targetTypeInfoTableLen = ARRAY_LEN(targetTypeInfoTable);
@@ -133,12 +125,11 @@ Bool NvCtrlIsTargetTypeValid(CtrlTargetType target_type)
         case X_SCREEN_TARGET:
         case GPU_TARGET:
         case FRAMELOCK_TARGET:
-        case VCS_TARGET:
-        case GVI_TARGET:
         case COOLER_TARGET:
         case THERMAL_SENSOR_TARGET:
         case NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET:
         case DISPLAY_TARGET:
+        case MUX_TARGET:
             return TRUE;
         default:
             return FALSE;
@@ -605,6 +596,7 @@ ReturnStatus NvCtrlQueryTargetCount(const CtrlTarget *ctrl_target,
                                     int *val)
 {
     const NvCtrlAttributePrivateHandle *h = getPrivateHandleConst(ctrl_target);
+    ReturnStatus ret = NvCtrlMissingExtension;
 
     if (h == NULL) {
         return NvCtrlBadHandle;
@@ -615,9 +607,9 @@ ReturnStatus NvCtrlQueryTargetCount(const CtrlTarget *ctrl_target,
         case THERMAL_SENSOR_TARGET:
         case COOLER_TARGET:
             {
-                ReturnStatus ret = NvCtrlNvmlQueryTargetCount(ctrl_target,
-                                                              target_type,
-                                                              val);
+                ret = NvCtrlNvmlQueryTargetCount(ctrl_target,
+                                                 target_type,
+                                                 val);
                 if ((ret != NvCtrlMissingExtension) &&
                     (ret != NvCtrlBadHandle) &&
                     (ret != NvCtrlNotSupported)) {
@@ -628,9 +620,15 @@ ReturnStatus NvCtrlQueryTargetCount(const CtrlTarget *ctrl_target,
         case DISPLAY_TARGET:
         case X_SCREEN_TARGET:
         case FRAMELOCK_TARGET:
-        case VCS_TARGET:
-        case GVI_TARGET:
         case NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET:
+        case MUX_TARGET:
+            if (!h->nv) {
+                /*
+                 * If there is no connection to the X Driver, return the
+                 * non-success value from the NVML query, if available.
+                 */
+                return ret;
+            }
             return NvCtrlNvControlQueryTargetCount(h, target_type, val);
         default:
             return NvCtrlBadHandle;
@@ -684,6 +682,7 @@ ReturnStatus NvCtrlGetAttributePerms(const CtrlTarget *ctrl_target,
                                      CtrlAttributePerms *perms)
 {
     const NvCtrlAttributePrivateHandle *h = getPrivateHandleConst(ctrl_target);
+    ReturnStatus ret = NvCtrlError;
 
     if (h == NULL) {
         return NvCtrlBadHandle;
@@ -698,10 +697,15 @@ ReturnStatus NvCtrlGetAttributePerms(const CtrlTarget *ctrl_target,
         case CTRL_ATTRIBUTE_TYPE_STRING:
         case CTRL_ATTRIBUTE_TYPE_BINARY_DATA:
         case CTRL_ATTRIBUTE_TYPE_STRING_OPERATION:
+
+            ret = NvCtrlNvmlGetAttributePerms(h, attr_type, attr, perms);
+
+            if (ret == NvCtrlSuccess || h->dpy == NULL) {
+                return ret;
+            }
             return NvCtrlNvControlGetAttributePerms(h, attr_type, attr, perms);
 
         case CTRL_ATTRIBUTE_TYPE_COLOR:
-        case CTRL_ATTRIBUTE_TYPE_SDI_CSC:
             /*
              * Allow non NV-CONTROL attributes to be read/written on X screen
              * targets
@@ -771,18 +775,17 @@ ReturnStatus NvCtrlGetDisplayAttribute64(const CtrlTarget *ctrl_target,
     if (((attr >= 0) && (attr <= NV_CTRL_LAST_ATTRIBUTE)) ||
         ((attr >= NV_CTRL_ATTR_NV_BASE) &&
          (attr <= NV_CTRL_ATTR_NV_LAST_ATTRIBUTE))) {
+        ReturnStatus ret = NvCtrlMissingExtension;
 
         switch (h->target_type) {
             case GPU_TARGET:
             case THERMAL_SENSOR_TARGET:
             case COOLER_TARGET:
                 {
-                    ReturnStatus ret = NvCtrlNvmlGetAttribute(ctrl_target,
-                                                              attr,
-                                                              val);
-                    if ((ret != NvCtrlMissingExtension) &&
-                        (ret != NvCtrlBadHandle) &&
-                        (ret != NvCtrlNotSupported)) {
+                    ret = NvCtrlNvmlGetAttribute(ctrl_target,
+                                                 attr,
+                                                 val);
+                    if (ret == NvCtrlSuccess) {
                         return ret;
                     }
                 }
@@ -790,10 +793,15 @@ ReturnStatus NvCtrlGetDisplayAttribute64(const CtrlTarget *ctrl_target,
             case DISPLAY_TARGET:
             case X_SCREEN_TARGET:
             case FRAMELOCK_TARGET:
-            case VCS_TARGET:
-            case GVI_TARGET:
             case NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET:
-                if (!h->nv) return NvCtrlMissingExtension;
+            case MUX_TARGET:
+                if (!h->nv) {
+                    /*
+                     * If there is no connection to the X Driver, return the
+                     * non-success value from the NVML query, if available.
+                     */
+                    return ret;
+                }
                 return NvCtrlNvControlGetAttribute(h, display_mask, attr, val);
             default:
                 return NvCtrlBadHandle;
@@ -827,6 +835,7 @@ ReturnStatus NvCtrlSetDisplayAttribute(CtrlTarget *ctrl_target,
                                        int attr, int val)
 {
     NvCtrlAttributePrivateHandle *h = getPrivateHandle(ctrl_target);
+    ReturnStatus ret = NvCtrlMissingExtension;
 
     if (h == NULL) {
         return NvCtrlBadHandle;
@@ -838,7 +847,6 @@ ReturnStatus NvCtrlSetDisplayAttribute(CtrlTarget *ctrl_target,
             case THERMAL_SENSOR_TARGET:
             case COOLER_TARGET:
                 {
-                    ReturnStatus ret;
                     ret = NvCtrlNvmlSetAttribute(ctrl_target,
                                                  attr,
                                                  display_mask,
@@ -853,11 +861,14 @@ ReturnStatus NvCtrlSetDisplayAttribute(CtrlTarget *ctrl_target,
             case DISPLAY_TARGET:
             case X_SCREEN_TARGET:
             case FRAMELOCK_TARGET:
-            case VCS_TARGET:
-            case GVI_TARGET:
             case NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET:
+            case MUX_TARGET:
                 if (!h->nv) {
-                    return NvCtrlMissingExtension;
+                    /*
+                     * If there is no connection to the X Driver, return the
+                     * non-success value from the NVML query, if available.
+                     */
+                    return ret;
                 }
                 return NvCtrlNvControlSetAttribute(h, display_mask, attr, val);
             default:
@@ -904,6 +915,7 @@ NvCtrlGetValidDisplayAttributeValues(const CtrlTarget *ctrl_target,
                                      CtrlAttributeValidValues *val)
 {
     const NvCtrlAttributePrivateHandle *h = getPrivateHandleConst(ctrl_target);
+    ReturnStatus ret = NvCtrlMissingExtension;
 
     if (h == NULL) {
         return NvCtrlBadHandle;
@@ -915,13 +927,10 @@ NvCtrlGetValidDisplayAttributeValues(const CtrlTarget *ctrl_target,
             case THERMAL_SENSOR_TARGET:
             case COOLER_TARGET:
                 {
-                    ReturnStatus ret;
                     ret = NvCtrlNvmlGetValidAttributeValues(ctrl_target,
                                                             attr,
                                                             val);
-                    if ((ret != NvCtrlMissingExtension) &&
-                        (ret != NvCtrlBadHandle) &&
-                        (ret != NvCtrlNotSupported)) {
+                    if (ret == NvCtrlSuccess) {
                         return ret;
                     }
                 }
@@ -929,11 +938,14 @@ NvCtrlGetValidDisplayAttributeValues(const CtrlTarget *ctrl_target,
             case DISPLAY_TARGET:
             case X_SCREEN_TARGET:
             case FRAMELOCK_TARGET:
-            case VCS_TARGET:
-            case GVI_TARGET:
             case NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET:
+            case MUX_TARGET:
                 if (!h->nv) {
-                    return NvCtrlMissingExtension;
+                    /*
+                     * If there is no connection to the X Driver, return the
+                     * non-success value from the NVML query, if available.
+                     */
+                    return ret;
                 }
                 return NvCtrlNvControlGetValidAttributeValues(h, display_mask,
                                                               attr, val);
@@ -949,7 +961,7 @@ NvCtrlGetValidDisplayAttributeValues(const CtrlTarget *ctrl_target,
 
 /*
  * GetValidStringDisplayAttributeValuesExtraAttr() -fill the
- * CtrlAttributeValidValues strucure for extra string atrributes i.e.
+ * CtrlAttributeValidValues strucure for extra string attributes i.e.
  * NvCtrlNvControl*, NvCtrlGlx*, NvCtrlXrandr*, NvCtrlVidMode*, or NvCtrlXv*.
  */
 
@@ -979,6 +991,7 @@ NvCtrlGetValidStringDisplayAttributeValues(const CtrlTarget *ctrl_target,
                                            CtrlAttributeValidValues *val)
 {
     const NvCtrlAttributePrivateHandle *h = getPrivateHandleConst(ctrl_target);
+    ReturnStatus ret = NvCtrlMissingExtension;
 
     if (h == NULL) {
         return NvCtrlBadHandle;
@@ -990,13 +1003,10 @@ NvCtrlGetValidStringDisplayAttributeValues(const CtrlTarget *ctrl_target,
             case THERMAL_SENSOR_TARGET:
             case COOLER_TARGET:
                 {
-                    ReturnStatus ret;
                     ret = NvCtrlNvmlGetValidStringAttributeValues(ctrl_target,
                                                                   attr,
                                                                   val);
-                    if ((ret != NvCtrlMissingExtension) &&
-                        (ret != NvCtrlBadHandle) &&
-                        (ret != NvCtrlNotSupported)) {
+                    if (ret == NvCtrlSuccess) {
                         return ret;
                     }
                 }
@@ -1004,11 +1014,14 @@ NvCtrlGetValidStringDisplayAttributeValues(const CtrlTarget *ctrl_target,
             case DISPLAY_TARGET:
             case X_SCREEN_TARGET:
             case FRAMELOCK_TARGET:
-            case VCS_TARGET:
-            case GVI_TARGET:
             case NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET:
+            case MUX_TARGET:
                 if (!h->nv) {
-                    return NvCtrlMissingExtension;
+                    /*
+                     * If there is no connection to the X Driver, return the
+                     * non-success value from the NVML query, if available.
+                     */
+                    return ret;
                 }
                 return NvCtrlNvControlGetValidStringDisplayAttributeValues(
                            h, display_mask, attr, val);
@@ -1089,9 +1102,8 @@ ReturnStatus NvCtrlGetStringDisplayAttribute(const CtrlTarget *ctrl_target,
         case DISPLAY_TARGET:
         case X_SCREEN_TARGET:
         case FRAMELOCK_TARGET:
-        case VCS_TARGET:
-        case GVI_TARGET:
         case NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET:
+        case MUX_TARGET:
             if ((attr >= 0) && (attr <= NV_CTRL_STRING_LAST_ATTRIBUTE)) {
                 if (!h->nv) return NvCtrlMissingExtension;
                 return NvCtrlNvControlGetStringAttribute(h, display_mask, attr, ptr);
@@ -1171,9 +1183,8 @@ ReturnStatus NvCtrlSetStringDisplayAttribute(CtrlTarget *ctrl_target,
             case DISPLAY_TARGET:
             case X_SCREEN_TARGET:
             case FRAMELOCK_TARGET:
-            case VCS_TARGET:
-            case GVI_TARGET:
             case NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET:
+            case MUX_TARGET:
                 if (!h->nv) return NvCtrlMissingExtension;
                 return NvCtrlNvControlSetStringAttribute(h, display_mask, attr,
                                                          ptr);
@@ -1191,6 +1202,7 @@ ReturnStatus NvCtrlGetBinaryAttribute(const CtrlTarget *ctrl_target,
                                       unsigned char **data, int *len)
 {
     const NvCtrlAttributePrivateHandle *h = getPrivateHandleConst(ctrl_target);
+    ReturnStatus ret = NvCtrlMissingExtension;
 
     if (h == NULL) {
         return NvCtrlBadHandle;
@@ -1201,10 +1213,10 @@ ReturnStatus NvCtrlGetBinaryAttribute(const CtrlTarget *ctrl_target,
         case THERMAL_SENSOR_TARGET:
         case COOLER_TARGET:
             {
-                ReturnStatus ret = NvCtrlNvmlGetBinaryAttribute(ctrl_target,
-                                                                attr,
-                                                                data,
-                                                                len);
+                ret = NvCtrlNvmlGetBinaryAttribute(ctrl_target,
+                                                   attr,
+                                                   data,
+                                                   len);
                 if ((ret != NvCtrlMissingExtension) &&
                     (ret != NvCtrlBadHandle) &&
                     (ret != NvCtrlNotSupported)) {
@@ -1215,9 +1227,14 @@ ReturnStatus NvCtrlGetBinaryAttribute(const CtrlTarget *ctrl_target,
         case DISPLAY_TARGET:
         case X_SCREEN_TARGET:
         case FRAMELOCK_TARGET:
-        case VCS_TARGET:
-        case GVI_TARGET:
         case NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET:
+            if (!h->nv) {
+                /*
+                 * If there is no connection to the X Driver, return the
+                 * non-success value from the NVML query, if available.
+                 */
+                return ret;
+            }
             return NvCtrlNvControlGetBinaryAttribute(h, display_mask, attr, data, len);
         default:
             return NvCtrlBadHandle;
@@ -1656,6 +1673,11 @@ NvCtrlEventHandle *NvCtrlGetEventHandle(const CtrlTarget *ctrl_target)
         return NULL;
     }
 
+    if (!h->dpy && !h->nv && h->nvml) {
+        /* We are running with NVML lib only. Events not yet supported.*/
+        return NULL;
+    }
+
     /* Look for the event handle */
     evt_h = NULL;
     for (evt_hnode = __event_handles;
