@@ -44,6 +44,7 @@ typedef enum {
     THERMAL_SENSOR_TARGET,
     NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET,
     DISPLAY_TARGET,
+    MUX_TARGET,
 
     MAX_TARGET_TYPES,
     INVALID_TARGET = -1
@@ -138,6 +139,7 @@ struct _CtrlSystem {
     char *display;  /* string for XOpenDisplay */
     Display *dpy;   /* X display connection */
     Bool has_nv_control;
+    Bool has_nvml;
 
     CtrlTargetNode *targets[MAX_TARGET_TYPES]; /* Shadows targetTypeTable */
     CtrlTargetNode *physical_screens;
@@ -573,6 +575,17 @@ typedef struct {
 #define NV_CTRL_STRING_XV_LAST_ATTRIBUTE  (NV_CTRL_STRING_XV_VERSION)
 
 
+/*
+ * Additional XVideo string attributes for NvCtrlGetStringDisplayAttribute();
+ */
+
+#define NV_CTRL_STRING_NVML_BASE            (NV_CTRL_STRING_XV_LAST_ATTRIBUTE + 1)
+
+#define NV_CTRL_STRING_NVML_VERSION         (NV_CTRL_STRING_NVML_BASE)
+
+#define NV_CTRL_STRING_NVML_LAST_ATTRIBUTE  (NV_CTRL_STRING_NVML_VERSION)
+
+
 
 #define NV_CTRL_ATTRIBUTES_NV_CONTROL_SUBSYSTEM   0x1
 #define NV_CTRL_ATTRIBUTES_XF86VIDMODE_SUBSYSTEM  0x2
