@@ -85,7 +85,8 @@
 #define TARGET_TYPE_IS_NVML_COMPATIBLE(_TARGET_TYPE_) \
     (((_TARGET_TYPE_) == GPU_TARGET) ||               \
      ((_TARGET_TYPE_) == THERMAL_SENSOR_TARGET) ||    \
-     ((_TARGET_TYPE_) == COOLER_TARGET))
+     ((_TARGET_TYPE_) == COOLER_TARGET) ||            \
+     ((_TARGET_TYPE_) == MUX_TARGET))
 
 #define TARGET_TYPE_NEEDS_NVCONTROL(_TARGET_TYPE_)     \
     (!(TARGET_TYPE_IS_NVML_COMPATIBLE(_TARGET_TYPE_)))
@@ -169,15 +170,31 @@ struct __NvCtrlNvmlAttributes {
         typeof(nvmlDeviceGetUUID)                       (*deviceGetUUID);
         typeof(nvmlDeviceGetCount)                      (*deviceGetCount);
         typeof(nvmlDeviceGetTemperature)                (*deviceGetTemperature);
-        typeof(nvmlDeviceGetFanSpeed)                   (*deviceGetFanSpeed);
         typeof(nvmlDeviceGetName)                       (*deviceGetName);
         typeof(nvmlDeviceGetVbiosVersion)               (*deviceGetVbiosVersion);
         typeof(nvmlDeviceGetMemoryInfo)                 (*deviceGetMemoryInfo);
+        typeof(nvmlDeviceGetMemoryInfo_v2)              (*deviceGetMemoryInfo_v2);
         typeof(nvmlDeviceGetPciInfo)                    (*deviceGetPciInfo);
+        typeof(nvmlDeviceGetCurrPcieLinkWidth)          (*deviceGetCurrPcieLinkWidth);
         typeof(nvmlDeviceGetMaxPcieLinkGeneration)      (*deviceGetMaxPcieLinkGeneration);
         typeof(nvmlDeviceGetMaxPcieLinkWidth)           (*deviceGetMaxPcieLinkWidth);
         typeof(nvmlDeviceGetVirtualizationMode)         (*deviceGetVirtualizationMode);
         typeof(nvmlDeviceGetGridLicensableFeatures_v4)  (*deviceGetGridLicensableFeatures);
+        typeof(nvmlDeviceGetUtilizationRates)           (*deviceGetUtilizationRates);
+        typeof(nvmlDeviceGetTemperatureThreshold)       (*deviceGetTemperatureThreshold);
+        typeof(nvmlDeviceGetFanSpeed_v2)                (*deviceGetFanSpeed_v2);
+        typeof(nvmlSystemGetDriverVersion)              (*systemGetDriverVersion);
+        typeof(nvmlDeviceGetEccMode)                    (*deviceGetEccMode);
+        typeof(nvmlDeviceSetEccMode)                    (*deviceSetEccMode);
+        typeof(nvmlDeviceGetTotalEccErrors)             (*deviceGetTotalEccErrors);
+        typeof(nvmlDeviceClearEccErrorCounts)           (*deviceClearEccErrorCounts);
+        typeof(nvmlDeviceGetMemoryErrorCounter)         (*deviceGetMemoryErrorCounter);
+        typeof(nvmlSystemGetNVMLVersion)                (*systemGetNVMLVersion);
+        typeof(nvmlDeviceGetNumGpuCores)                (*deviceGetNumGpuCores);
+        typeof(nvmlDeviceGetMemoryBusWidth)             (*deviceGetMemoryBusWidth);
+        typeof(nvmlDeviceGetIrqNum)                     (*deviceGetIrqNum);
+        typeof(nvmlDeviceGetPowerSource)                (*deviceGetPowerSource);
+        typeof(nvmlDeviceGetNumFans)                    (*deviceGetNumFans);
 
     } lib;
 
@@ -456,4 +473,9 @@ NvCtrlNvmlGetValidAttributeValues(const CtrlTarget *ctrl_target,
                                   int attr,
                                   CtrlAttributeValidValues *val);
 
+ReturnStatus
+NvCtrlNvmlGetAttributePerms(const NvCtrlAttributePrivateHandle *,
+                            CtrlAttributeType, int,
+                            CtrlAttributePerms *);
+
 #endif /* __NVCTRL_ATTRIBUTES_PRIVATE__ */
